@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017 Nicolas Mora <mail@babelouest.org>
+ * Copyright (c) 2017-2020 Nicolas Mora <mail@babelouest.org>
  * Copyright (c) 2009-2014 Petri Lehtinen <petri@digip.org>
  * Copyright (c) 2011-2012 Basile Starynkevitch <basile@starynkevitch.net>
  *
@@ -9,7 +9,7 @@
  */
 
 #include <stdlib.h>
-#include "../include/orcania.h"
+#include "orcania.h"
 
 /* memory function pointers */
 static o_malloc_t do_malloc = malloc;
@@ -44,9 +44,6 @@ void o_set_alloc_funcs(o_malloc_t malloc_fn, o_realloc_t realloc_fn, o_free_t fr
   do_malloc = malloc_fn;
   do_realloc = realloc_fn;
   do_free = free_fn;
-#ifndef U_DISABLE_JANSSON
-  json_set_alloc_funcs(malloc_fn, free_fn);
-#endif
 }
 
 void o_get_alloc_funcs(o_malloc_t * malloc_fn, o_realloc_t * realloc_fn, o_free_t * free_fn) {
